package org.wso2.carbon.application.deployer.config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

/**
 * Represents a single component in the application.xml. In other words, keeps information
 * about a single artifact deployed through an application.
 */
public class Component {

    private static final Log log = LogFactory.getLog(Component.class);

    public static final String COMPONENT = "Component";
    public static final String ID = "Id";
    public static final String TYPE = "Type";
    public static final String VERSION = "Version";
    public static final String DEPLOYABLE_ARTIFACT = "DeployableArtifact";

    public static final String RESOURCES = "Resources";
    public static final String ITEM = "Item";
    public static final String COLLECTION = "Collection";
    public static final String ASSOCIATION = "Association";
    public static final String PATH = "Path";
    public static final String CONTENT = "Content";
    public static final String DIRECTORY = "Directory";
    public static final String SOURCE_PATH = "SourcePath";
    public static final String TARGET_PATH = "TargetPath";

    private String id;
    private String type;
    private String version;
    private String artifactName;
    private HashMap<String, String> resources;
    private HashMap<String, String> collections;
    private List<Association> associations;

    public Component() {
        resources = new HashMap<String, String>();
        collections = new HashMap<String, String>();
        associations = new ArrayList<Association>();
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getArtifactName() {
        return artifactName;
    }

    public void setArtifactName(String artifactName) {
        this.artifactName = artifactName;
    }

    public HashMap<String, String> getResources() {
        return resources;
    }

    /**
     * Add a resource which belongs to the artifact represented by this component.
     *
     * @param path    - path at which this resource should be stored
     * @param content - name of the file to store as a resource
     */
    public void addResource(String path, String content) {
        if (path == null || path.length() == 0) {
            log.error("Resource path not found");
            return;
        }
        if (content == null || content.length() == 0) {
            log.error("Content name of the resource not found");
            return;
        }
        resources.put(path, content);
    }

    public HashMap<String, String> getCollections() {
        return collections;
    }

    /**
     * Add a directory and all it's contents which belongs to the artifact represented by
     * this component.
     *
     * @param path      - path at which this collection should be stored
     * @param directory - name of the root directory to store as a resource
     */
    public void addCollection(String path, String directory) {
        if (path == null || path.length() == 0) {
            log.error("Collection path not found");
            return;
        }
        if (directory == null || directory.length() == 0) {
            log.error("Directory name not found");
            return;
        }
        collections.put(path, directory);
    }

    public List<Association> getAssociations() {
        return associations;
    }

    /**
     * Add an association which should is needed to properly deploy the artifact represented
     * by this component
     *
     * @param a - Association instance
     */
    public void addAssociation(Association a) {
        associations.add(a);
    }
}
