package org.wso2.carbon.application.deployer.internal;

import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.application.deployer.*;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @scr.component name="application.deployer.dscomponent" immediate="true"
 * @scr.reference name="registry.service" interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic" bind="setRegistryService" unbind="unsetRegistryService"
 */

public class AppDeployerServiceComponent {

    private static RegistryService registryService;

    private ApplicationManager applicationManager = null;

    private static final Log log = LogFactory.getLog(AppDeployerServiceComponent.class);

    protected void activate(ComponentContext ctxt) {
        try {
            applicationManager = new ApplicationManager(ctxt.getBundleContext());

            //register ApplicationManager as a service
            ctxt.getBundleContext().registerService(ApplicationManager.class.getName(),
                    applicationManager, null);

            log.debug("Carbon Application Deployer is activated..");

        } catch (Throwable e) {
            log.error("Failed to activate Carbon Application Deployer", e);
        }
    }

    protected void deactivate(ComponentContext ctxt) {
        if (applicationManager != null) {
            ctxt.getBundleContext().removeBundleListener(applicationManager.getBundleListener());
        }
    }

    protected void setRegistryService(RegistryService regService) {
        registryService = regService;
    }

    protected void unsetRegistryService(RegistryService regService) {
        registryService = null;
    }

    public static RegistryService getRegistryService() throws Exception {
        if (registryService == null) {
            String msg = "Before activating Carbon Application deployer bundle, an instance of "
                    + "RegistryService should be in existance";
            log.error(msg);
            throw new Exception(msg);
        }
        return registryService;
    }

}
