/*
*  Copyright (c) 2005-2010, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/

package org.wso2.carbon.application.deployer;

import org.apache.axis2.deployment.Deployer;
import org.apache.axis2.deployment.DeploymentException;
import org.apache.axis2.deployment.repository.util.DeploymentFileData;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.application.deployer.internal.ApplicationManager;

/**
 * This is the Axis2 deployer which is resoponsible for cApp deployment.
 */
public class CappAxis2Deployer implements Deployer {

    private static final Log log = LogFactory.getLog(CappAxis2Deployer.class);

    public void init(ConfigurationContext configurationContext) {
        if(log.isDebugEnabled()) {
            log.debug("Initializing Capp Axis2 Deployer...");
        }
    }

    /**
     * Axis2 deployment engine will call this method when a .car archive is deployed. So
     * we only have to call the applicationManager to deploy it using the absolute path of
     * the deployed .car file.
     * IMPORTANT :
     * We have to use a separate thread for this. car deployment will take some time as it
     * involves a feature installation through p2. If it takes more than 10 seconds, Axis2
     * deployment engine will malfunction as the current thread will be blocked. To avoid
     * that, we use a separate thread to deploy the car and the axis2 thread is released.
     *
     * @param deploymentFileData - info about the deployed file
     * @throws DeploymentException - error while deploying cApp
     */
    public void deploy(DeploymentFileData deploymentFileData) throws DeploymentException {
        CappDeploymentWorker deploymentWorker = new
                CappDeploymentWorker(deploymentFileData.getAbsolutePath());
        new Thread(deploymentWorker).start();
    }

    public void setDirectory(String s) {

    }

    public void setExtension(String s) {

    }

    /**
     * Undeploys the cApp from system when the .car file is deleted from the repository. Find
     * the relevant cApp using the file path and call the undeploy method on applicationManager.
     * IMPORTANT:
     * We have to use a separate thread for this. car undeployment will take some time as it
     * involves a feature uninstallation through p2. If it takes more than 10 seconds, Axis2
     * deployment engine will malfunction as the current thread will be blocked. To avoid
     * that, we use a separate thread to undeploy the car and the axis2 thread is released.
     *
     * @param filePath - deleted .car file path
     * @throws DeploymentException
     */
    public void unDeploy(String filePath) throws DeploymentException {
        CappUndeploymentWorker undeploymentWorker = new CappUndeploymentWorker(filePath);
        new Thread(undeploymentWorker).start();
    }

    private static class CappDeploymentWorker implements Runnable {

        private String artifactPath;

        public CappDeploymentWorker(String path) {
            this.artifactPath = path;
        }

        public void run() {
            try {
                ApplicationManager.getInstance().deployCarbonApp(artifactPath);
            } catch (Exception e) {
                log.error("Error while deploying carbon application " + artifactPath, e);
            }
        }
    }

    private static class CappUndeploymentWorker implements Runnable {

        private String artifactPath;

        public CappUndeploymentWorker(String path) {
            this.artifactPath = path;
        }

        public void run() {
            artifactPath = AppDeployerUtils.formatPath(artifactPath);
            CarbonApplication existingApp = null;
            for (CarbonApplication carbonApp : ApplicationManager.getInstance().getCarbonApps()) {
                if (artifactPath.equals(carbonApp.getAppFilePath())) {
                    existingApp = carbonApp;
                    break;
                }
            }
            if (existingApp != null) {
                ApplicationManager.getInstance().undeployCarbonApp(existingApp);
            }
        }
    }

}
