/*
*  Copyright (c) 2005-2010, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/

package org.wso2.carbon.application.deployer.config;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

/**
 * This will hold the registry resources config file declarations. This can include resources,
 * collections and associations
 */
public class RegistryConfig {

    private Log log = LogFactory.getLog(RegistryConfig.class);

    private HashMap<String, String> resources;
    private HashMap<String, String> collections;
    private List<Association> associations;

    private String extractedPath;
    private String parentArtifactName;
    private String configFileName;

    public RegistryConfig() {
        resources = new HashMap<String, String>();
        collections = new HashMap<String, String>();
        associations = new ArrayList<Association>();
    }

    /**
     * Add a resource which belongs to the artifact represented by this component.
     *
     * @param path    - path at which this resource should be stored
     * @param content - name of the file to store as a resource
     */
    public void addResource(String path, String content) {
        if (path == null || path.length() == 0) {
            log.error("Resource path not found");
            return;
        }
        if (content == null || content.length() == 0) {
            log.error("Content name of the resource not found");
            return;
        }
        resources.put(path, content);
    }

    public HashMap<String, String> getCollections() {
        return collections;
    }

    /**
     * Add a directory and all it's contents which belongs to the artifact represented by
     * this component.
     *
     * @param path      - path at which this collection should be stored
     * @param directory - name of the root directory to store as a resource
     */
    public void addCollection(String path, String directory) {
        if (path == null || path.length() == 0) {
            log.error("Collection path not found");
            return;
        }
        if (directory == null || directory.length() == 0) {
            log.error("Directory name not found");
            return;
        }
        collections.put(path, directory);
    }

    public List<Association> getAssociations() {
        return associations;
    }

    /**
     * Add an association which should is needed to properly deploy the artifact represented
     * by this component
     *
     * @param a - Association instance
     */
    public void addAssociation(Association a) {
        associations.add(a);
    }

    public HashMap<String, String> getResources() {
        return resources;
    }

    public String getExtractedPath() {
        return extractedPath;
    }

    public void setExtractedPath(String extractedPath) {
        this.extractedPath = extractedPath;
    }

    public String getParentArtifactName() {
        return parentArtifactName;
    }

    public void setParentArtifactName(String parentArtifactName) {
        this.parentArtifactName = parentArtifactName;
    }

    public String getConfigFileName() {
        return configFileName;
    }

    public void setConfigFileName(String configFileName) {
        this.configFileName = configFileName;
    }
}
