/*
*  Copyright (c) 2005-2010, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/

package org.wso2.carbon.application.deployer.handler;

import org.wso2.carbon.application.deployer.CarbonApplication;
import org.wso2.carbon.application.deployer.AppDeployerConstants;
import org.wso2.carbon.application.deployer.persistence.CarbonAppPersistenceManager;
import org.wso2.carbon.application.deployer.config.ApplicationConfiguration;
import org.wso2.carbon.application.deployer.config.Artifact;
import org.wso2.carbon.application.deployer.config.RegistryConfig;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.List;
import java.util.ArrayList;

public class RegistryResourceUndeployer implements AppUndeploymentHandler {

    private static final Log log = LogFactory.getLog(RegistryResourceUndeployer.class);

    public void undeployArtifacts(CarbonApplication carbonApp) {
        if (log.isDebugEnabled()) {
            log.debug("Registry resource Undeployer undeploying artifacts of cApp: " +
                    carbonApp.getAppName());
        }

        ApplicationConfiguration appConfig = carbonApp.getAppConfig();
        List<Artifact.Dependency> deps = appConfig.getApplicationArtifact().getDependencies();
        
        List<Artifact> artifacts = new ArrayList<Artifact>();
        for (Artifact.Dependency dep : deps) {
            if (dep.getArtifact() != null) {
                artifacts.add(dep.getArtifact());
            }
        }

        // undeploying registry resources in all dependent artifacts
        undeployRegistryArtifacts(artifacts, carbonApp.getAppName());
    }

    /**
     * Uneploys registry artifacts recursively. A Registry artifact can exist as a sub artifact in
     * any type of artifact. Therefore, have to search recursively
     *
     * @param artifacts - list of artifacts to be undeployed
     * @param parentAppName - name of the parent app name
     */
    private void undeployRegistryArtifacts(List<Artifact> artifacts, String parentAppName) {
        for (Artifact artifact : artifacts) {
            if (RegistryResourceDeployer.REGISTRY_RESOURCE_TYPE.equals(artifact.getType())) {
                try {
                    RegistryConfig regConfig = CarbonAppPersistenceManager.loadRegistryConfig(
                            AppDeployerConstants.APPLICATIONS + parentAppName +
                                    AppDeployerConstants.APP_DEPENDENCIES + artifact.getName());
                    if (regConfig != null) {
                        CarbonAppPersistenceManager.removeArtifactResources(regConfig);
                    }
                } catch (Exception e) {
                    log.error("Error while loading registry config for artifact " +
                            artifact.getName(), e);
                }
            }
            // set the parent name for all sub artifacts..
            List<Artifact> subArtifacts = artifact.getSubArtifacts();
            if (subArtifacts.size() != 0) {
                for (Artifact sub : subArtifacts) {
                    sub.setName(artifact.getName());
                }
            }
            undeployRegistryArtifacts(subArtifacts, parentAppName);
        }
    }
}