/*
*  Copyright (c) 2005-2010, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/

package org.wso2.carbon.application.deployer.service;

import org.wso2.carbon.application.deployer.handler.AppDeploymentHandler;
import org.wso2.carbon.application.deployer.handler.AppUndeploymentHandler;
import org.wso2.carbon.application.deployer.CarbonApplication;

import java.util.ArrayList;


/**
 * OSGi service interface for the Application Management
 */
public interface ApplicationManagerService {

    /**
     * All app deployers register their instances throgh this method
     *
     * @param handler - app deployer which implements the AppDeploymentHandler interface
     */
    public void registerDeploymentHandler(AppDeploymentHandler handler);

    /**
     * All app undeployers register their instances throgh this method
     *
     * @param handler - app undeployer which implements the AppUndeploymentHandler interface
     */
    public void registerUndeploymentHandler(AppUndeploymentHandler handler);

    /**
     * Unregister the specified handler if it is already regitered
     *
     * @param handler - input deployer handler
     */
    public void unregisterDeploymentHandler(AppDeploymentHandler handler);

    /**
     * Unregister the specified handler if it is already regitered
     *
     * @param handler - input undeployer handler
     */
    public void unregisterUndeploymentHandler(AppUndeploymentHandler handler);

    /**
     * Deploy the provided carbon App..
     *
     * @param archPath - carbon app archive path
     * @throws Exception - error on registry actions
     */
    public void deployCarbonApp(String archPath) throws Exception;

    /**
     * Undeploy the provided carbon App..
     *
     * @param carbonApp - CarbonApplication instance
     */
    public void undeployCarbonApp(CarbonApplication carbonApp);

    /**
     * Returns all deployed carbon apps in the system
     *
     * @return List of carbon apps
     */
    public ArrayList<CarbonApplication> getCarbonApps();

}
