package org.wso2.carbon.core.services.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.core.services.authentication.stats.LoginAttempt;
import org.wso2.carbon.core.services.authentication.stats.LoginStatDatabase;
import org.wso2.carbon.core.services.callback.LoginSubscriptionManagerServiceImpl;
import org.wso2.carbon.core.services.internal.CarbonServicesServiceComponent;
import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.registry.core.session.UserRegistry;
import org.wso2.carbon.utils.ServerConstants;
import org.wso2.carbon.utils.WSO2Constants;

import javax.servlet.http.HttpSession;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class CarbonAuthenticationUtil {

    private static final Log log = LogFactory.getLog(CarbonAuthenticationUtil.class);
    
    public static void onFailedAdminLogin(HttpSession httpSess, String username, int tenantId,
            String remoteAddress, String reason) throws Exception {

        Date currentTime = Calendar.getInstance().getTime();
        SimpleDateFormat date = new SimpleDateFormat("'['yyyy-MM-dd HH:mm:ss,SSSS']'");

        log.warn("Failed Administrator login attempt \'" + username + "\' at "
                + date.format(currentTime) + " from IP address " + remoteAddress);

        if (httpSess != null) {
            httpSess.invalidate();
        }
        LoginAttempt loginAttempt = new LoginAttempt(username, tenantId, remoteAddress, new Date(),
                false, reason);
        LoginStatDatabase.recordLoginAttempt(loginAttempt);
    }

    public static void onSuccessAdminLogin(HttpSession httpSess, String username, int tenantId,
            String tenantDomain, String remoteAddress) throws Exception {

        Date currentTime = Calendar.getInstance().getTime();
        SimpleDateFormat date = new SimpleDateFormat("'['yyyy-MM-dd HH:mm:ss,SSSS']'");

        log.info("\'" + username + "\' logged in at " + date.format(currentTime)
                + " from IP address " + remoteAddress);
        
        
        UserRegistry userRegistry = CarbonServicesServiceComponent.getRegistryService()
                .getConfigUserRegistry(username, tenantId);

        UserRegistry systemRegistry = CarbonServicesServiceComponent.getRegistryService()
                .getConfigSystemRegistry(tenantId);
        if (httpSess != null) {
            httpSess.setAttribute(RegistryConstants.USER_REGISTRY, userRegistry);
            httpSess.setAttribute(RegistryConstants.SYSTEM_REGISTRY, systemRegistry);
            httpSess.setAttribute(ServerConstants.ADMIN_LOGGED_IN, "true");
            httpSess.setAttribute(ServerConstants.ADMIN_USERNAME, username);
            httpSess.setAttribute(RegistryConstants.TENANT_ID, new Integer(tenantId));
            httpSess.setAttribute(RegistryConstants.TENANT_DOMAIN, tenantDomain);
        }

        RegistryService registryService = CarbonServicesServiceComponent.getRegistryService();
        if (httpSess != null) {
            httpSess.setAttribute(WSO2Constants.GOVERNANCE_REGISTRY_INSTANCE, registryService
                    .getGovernanceUserRegistry(username, tenantId));
            httpSess.setAttribute(WSO2Constants.CONFIG_USER_REGISTRY_INSTANCE, registryService
                    .getConfigUserRegistry(username, tenantId));
            httpSess.setAttribute(RegistryConstants.ROOT_REGISTRY_INSTANCE, registryService
                    .getRegistry(username, tenantId));
        }

        // trigger the callbacks subscribe to the login event
        LoginSubscriptionManagerServiceImpl loginSubscriptionManagerServiceImpl = CarbonServicesServiceComponent
                .getLoginSubscriptionManagerServiceImpl();
        UserRegistry configRegistry = CarbonServicesServiceComponent.getRegistryService()
                .getConfigSystemRegistry(tenantId);
        loginSubscriptionManagerServiceImpl.triggerEvent(configRegistry, username, tenantId);

        if (log.isDebugEnabled()) {
            log.debug("User Registry instance is set in the session for user " + username);
        }
        LoginAttempt loginAttempt = new LoginAttempt(username, tenantId, remoteAddress, new Date(),
                true, null);
        LoginStatDatabase.recordLoginAttempt(loginAttempt);
    }

}
