/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core;

import java.util.HashMap;
import java.util.Map;

/**
 * Contains a resource path and tagging details of tags on that resource. Instances of this class is
 * returned from the tag searches. Tag searches may be performed on multiple tags. This class only
 * contains the tag counts for the tags that included in the search. e.g. If the search is for tags
 * java, jsp and programming and if the resource /books/computer/Java Web Development matches the
 * search, counts are only contained for the tags java, jsp and programming although that resource
 * may have many other tags like internet, web, servlets.
 */
public class TaggedResourcePath {

    /**
     * Resource path for which the tag counts are associated.
     */
    private String resourcePath;

    /**
     * Tag counts in the form tag -> count Where tag is a string and count is a long.
     */
    private Map<String, String> tagCounts = new HashMap<String, String>();

    /**
     * This is deprecated. Included only for backward compatibility.
     */
    private long tagCount;

    /**
     * Method to get the resource path.
     *
     * @return the resource path.
     */
    public String getResourcePath() {
        return resourcePath;
    }

    /**
     * Method to get the resource path.
     *
     * @param resourcePath the resource path.
     */
    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    /**
     * Method to get the tag count.
     *
     * @return the tag count.
     */
    public long getTagCount() {
        return tagCount;
    }

    /**
     * Method to set the tag count.
     *
     * @param tagCount the tag count.
     */
    public void setTagCount(long tagCount) {
        this.tagCount = tagCount;
    }

    /**
     * Method to get the tag count.
     *
     * @return the tag count.
     */
    public Map<String, String> getTagCounts() {
        return tagCounts;
    }

    /**
     * Method to set the tag count.
     *
     * @param tagCounts the tag count.
     */
    public void setTagCounts(Map<String, String> tagCounts) {
        this.tagCounts = tagCounts;
    }

    /**
     * Method to add the tag count.
     *
     * @param tag   the tag.
     * @param count the count.
     */
    public void addTagCount(String tag, long count) {
        tagCounts.put(tag, Long.toString(count));
    }
}
