package org.wso2.carbon.registry.core.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.jdbc.dao.LogsDAO;
import org.wso2.carbon.registry.core.session.CurrentSession;

import java.sql.Timestamp;

/**
 * Writes log records to the database on a separate thread
 */
public class LogWriter extends Thread {

    private static final Log log = LogFactory.getLog(LogWriter.class);

    private LogQueue logQueue = null;

    /**
     * Constructor accepting a queue of logs.
     * @param logQueue the queue of logs.
     */
    public LogWriter(LogQueue logQueue) {
        this.logQueue = logQueue;
    }

    /**
     * Sets the queue for holding and retrieving logs.
     * @param logQueue the queue for holding and retrieving logs.
     */
    @SuppressWarnings("unused")
    public void setLogQueue(LogQueue logQueue) {
        this.logQueue = logQueue;
    }

    /**
     * Obtains the queue for holding and retrieving logs.
     * @return queue for holding and retrieving logs.
     */
    public LogQueue getLogQueue() {
        return logQueue;
    }

    /**
     * Starts writing logs.
     */
    public synchronized void start() {
        super.start();
    }

    /**
     * The main business logic.
     */
    @SuppressWarnings("InfiniteLoopStatement")
    public void run() {
        while (true) {
            if (logQueue != null && !logQueue.isEmpty()) {

                int queueLength = logQueue.size();
                LogRecord[] logRecords = new LogRecord[queueLength];
                for (int a = 0; a < queueLength; a++) {
                    LogRecord logRecord = (LogRecord) logQueue.poll();
                    logRecords[a] = logRecord;
                }
                LogsDAO logsDAO = new LogsDAO();
                try {
                    logsDAO.saveLogBatch(logRecords);
                } catch (RegistryException e) {
                    //TODO: sumedha
                    //print failed log entries
                    log.error(e);
                }
            } else if (isInterrupted()) {
                break;
            }

            try {
                sleep(10 * 1000);
            } catch (InterruptedException e) {
                if (logQueue == null || logQueue.isEmpty()) {
                    break;
                }
            }
        }
    }


    /**
     * Adds log record to queue
     *
     * @param resourcePath the resource path.
     * @param userName     the name of the user who performed the action on the resource.
     * @param action       an identifier of the action that happened.
     * @param actionData   the data for further information.
     *
     * @throws RegistryException if the operation failed.
     */
    public void addLog(String resourcePath, String userName, int action,
                              String actionData) throws RegistryException {
        if (logQueue != null) {
            LogRecord logRecord = new LogRecord();
            if (CurrentSession.getLocalPathMap() != null) {
                String temp = CurrentSession.getLocalPathMap().get(resourcePath);
                if (temp != null) {
                    resourcePath = temp;
                }
            }
            logRecord.setResourcePath(resourcePath);
            logRecord.setUserName(userName);
            logRecord
                    .setTimestamp(new Timestamp(System.currentTimeMillis()));
            logRecord.setAction(action);
            logRecord.setActionData(actionData);
            logRecord.setTenantId(CurrentSession.getTenantId());

            logQueue.add(logRecord);
        }
	}
}
