package org.wso2.carbon.registry.dump;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.namespace.QName;
import java.io.*;
import java.net.URLEncoder;


public class DumpUtils {

    private static Log log = LogFactory.getLog(DumpUtils.class);

    private final static String ROOT_PATH = "/";
    private final static String PATH_SEPARATOR = "/";

    public static String getResourceName(String resourcePath) {
        String resourceName;
        if (resourcePath.equals(ROOT_PATH)) {
            resourceName = ROOT_PATH;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(PATH_SEPARATOR) == 0) {
                resourceName = formattedPath.substring(1, formattedPath.length());
            } else {
                resourceName = formattedPath.substring(
                        formattedPath.lastIndexOf(PATH_SEPARATOR) + 1,
                        formattedPath.length());
            }
        }
        return resourceName;
    }

    /**
     * creating the file that store the meta data of the current directory or file
     * @param fileName
     * @param metaData
     * @param modifiedDateAtCO
     */
    public static void createMetaFile(String fileName,
                                      OMElement metaData,
                                      long modifiedDateAtCO) {
        metaData.addAttribute("physicalModifiedDate", Long.toString(modifiedDateAtCO), null);

        try {
            File file = new File(fileName);

            String metaDirName = file.getParent();
            if (metaDirName != null) {
                File metaDir = new File(metaDirName);
                if (!metaDir.exists()) {
                    metaDir.mkdir();
                }
            }
            file.createNewFile();

            FileOutputStream fileOut = new FileOutputStream(file, false);
            metaData.serialize(fileOut);

            fileOut.close();

        } catch (IOException e) {
            log.error("Error in creating the meta file: " + fileName);
        } catch (XMLStreamException e) {
            log.error("Error in writing to the meta file: " + fileName);
        }

    }


    public static OMElement getOMElmentFromMetafile(String metafilename,
                                                    boolean isCollection,
                                                    String path) throws Exception {
        File metafile = new File(metafilename);
        OMElement element;
        // if the file exists, just get it.
        if (metafile.exists()) {
            try {
                FileInputStream directoryMetaFileStream = new FileInputStream(metafilename);

                //create the parser
                XMLStreamReader parser = XMLInputFactory.newInstance()
                        .createXMLStreamReader(directoryMetaFileStream);
                //create the builder
                StAXOMBuilder builder = new StAXOMBuilder(parser);
                // get the element to restore
                element = builder.getDocumentElement().cloneOMElement();

                if (path != null && isCollection) {
                    // normally in collection the paths are not renamed.. so doing it explictely
                    element.addAttribute("path", path, null);
                }

            } catch (Exception e) {
                throw new Exception("Error in reading the meta file: " + metafilename, e);
            }
            return element;
        }

        // if not provide the default details
        OMFactory factory = OMAbstractFactory.getOMFactory();
        OMElement root = factory.createOMElement(new QName("resource"));

        // adding path as an attribute
        root.addAttribute("path", path, null);

        // adding isCollection as an attribute
        root.addAttribute("isCollection", isCollection ? "true" : "false", null);
        OMElement child;

        // set createdTime
        long now = System.currentTimeMillis();
        child = factory.createOMElement(new QName("createdTime"));
        child.setText(Long.toString(now));
        root.addChild(child);

        // set LastModified
        child = factory.createOMElement(new QName("lastModified"));
        child.setText(Long.toString(now));
        root.addChild(child);

        // set Description
        child = factory.createOMElement(new QName("description"));
        root.addChild(child);

        return root;
    }


    /**
     * copying hte file to another
     *
     * @param src
     * @param dst
     * @throws Exception
     */
    public static void copy(File src, File dst) throws Exception {
        try {
            InputStream in = new FileInputStream(src);
            OutputStream out = new FileOutputStream(dst);

            // Transfer bytes from in to out
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            in.close();
            out.close();
        } catch (IOException e) {
            throw new Exception("Error in copying " + src + " to " + dst, e);
        }
    }

    public static String encodeResourcename(String resourceName) throws Exception {
        String encodedName;
        try {
            encodedName = URLEncoder.encode(resourceName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new Exception("Problem in encoding the resource path: " + resourceName + ". ", e);
        }
        return encodedName;
    }

    public static boolean deleteFile(File file) {
        if (file.isDirectory()) {
            String[] children = file.list();
            for (String aChildren : children) {
                boolean success = deleteFile(new File(file, aChildren));
                if (!success) {
                    return false;
                }
            }
        }
        return file.delete();
    }
}
