package org.wso2.carbon.server.admin.auth;

import java.util.Arrays;

import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;
import org.wso2.carbon.core.services.authentication.CarbonServerAuthenticator;

public class AuthenticatorServerRegistry {

    private static Log log = LogFactory.getLog(AuthenticatorServerRegistry.class);
    private static ServiceTracker authTracker;

    public static final String AUTHENTICATOR_TYPE = "authenticator.type";
    
    public static void init(BundleContext bc) throws Exception {
        try {
            authTracker = new ServiceTracker(bc, CarbonServerAuthenticator.class.getName(), null);
            authTracker.open();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw e;
        }
    }

    public static CarbonServerAuthenticator getCarbonAuthenticator(MessageContext msgContext) {
    	CarbonServerAuthenticator authenticator = null;
        Object[] objects = authTracker.getServices();
        // cast each object - cannot cast object array
        CarbonServerAuthenticator[] authenticators = new CarbonServerAuthenticator[objects.length];
        int i = 0;
        for (Object obj : objects) {
            authenticators[i] = (CarbonServerAuthenticator) obj;
            i++;
        }
        Arrays.sort(authenticators, new AuthenticatorComparator());
        for (CarbonServerAuthenticator auth : authenticators) {
            if (!auth.isDisabled() && auth.isHandle(msgContext)) {
                authenticator = auth;
                break;
            }
        }
        return authenticator;
    }
}
