package org.wso2.carbon.server.admin.internal;

import java.util.Hashtable;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.osgi.framework.console.CommandProvider;
import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.core.init.CarbonInitConstant;
import org.wso2.carbon.core.services.authentication.AuthenticationAdmin;
import org.wso2.carbon.core.services.authentication.CarbonServerAuthenticator;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.server.admin.auth.AuthenticatorServerRegistry;
import org.wso2.carbon.server.admin.common.IServerAdmin;
import org.wso2.carbon.server.admin.service.ServerAdmin;
import org.wso2.carbon.user.core.service.RealmService;
import org.wso2.carbon.utils.ConfigurationContextService;
import org.wso2.carbon.utils.MBeanRegistrar;
import org.wso2.carbon.utils.ServerConfiguration;

/**
 * @scr.component name="serveradmin.service.component"" immediate="true"
 * @scr.reference name="registry.service"
 * interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic"  bind="setRegistryService" unbind="unsetRegistryService"
 * @scr.reference name="config.context.service"
 * interface="org.wso2.carbon.utils.ConfigurationContextService"
 * cardinality="1..1" policy="dynamic"  bind="setConfigurationContextService" unbind="unsetConfigurationContextService"
 * @scr.reference name="server.configuration"
 * interface="org.wso2.carbon.utils.ServerConfiguration"
 * cardinality="1..1" policy="dynamic" bind="setServerConfiguration" unbind="unsetServerConfiguration"
 * @scr.reference name="user.realmservice.default" interface="org.wso2.carbon.user.core.service.RealmService"
 * cardinality="1..1" policy="dynamic" bind="setRealmService"
 * unbind="unsetRealmService"
 */
public class ServerAdminServiceComponent {

    private static final Log log = LogFactory.getLog(ServerAdminServiceComponent.class);

    private static String registryDriver;
    private static String usermanagerDriver;
    private boolean registeredMBeans;
    public static final String SERVER_ADMIN_MODULE_NAME = "ServerAdminModule";
    private ConfigurationContext configContext;
    private static ServerConfiguration serverConfig;
    private static RegistryService registryService;
    private static RealmService realmService;

    protected void activate(ComponentContext ctxt) {
        try {
            //Register ServerAdmin MBean
            registerMBeans(serverConfig);
            // Engaging ServerAdmin as an global module
            configContext.getAxisConfiguration().engageModule(SERVER_ADMIN_MODULE_NAME);

            //setUserManagerDriver(userRealmDefault);
            setRegistryDriver(registryService);
            
            ServerAdmin.classloader = Thread.currentThread().getContextClassLoader();
            ServerAdmin.serverConfig = serverConfig;
            ctxt.getBundleContext().registerService(IServerAdmin.class.getName(),
                                                    new ServerAdmin(),
                                                    null);
            ctxt.getBundleContext().registerService(CommandProvider.class.getName(),
                                                    new ServerAdminCommandProvider(),
                                                    null);
            AuthenticationAdmin authenticator = new AuthenticationAdmin();
            Hashtable<String, String> props = new Hashtable<String, String>();
            props.put(AuthenticatorServerRegistry.AUTHENTICATOR_TYPE, authenticator.getAuthenticatorName());
            ctxt.getBundleContext().registerService(CarbonServerAuthenticator.class.getName(), authenticator, props);

            AuthenticatorServerRegistry.init(ctxt.getBundleContext());
            log.debug("ServerAdmin bundle is activated");
        } catch (Throwable e) {
            log.error("Failed to activate ServerAdmin bundle", e);
        }
    }

    protected void deactivate(ComponentContext ctxt) {
        log.debug("ServerAdmin bundle is deactivated");
    }

    private void registerMBeans(ServerConfiguration serverConfig) {
        if (serverConfig.getFirstProperty(CarbonInitConstant.JMX_RMI_REGISTRY_CONF) != null) {
            if (registeredMBeans) return;
            MBeanRegistrar.registerMBean(new ServerAdmin());
            registeredMBeans = true;
        }
    }

    private void setRegistryDriver(RegistryService registry) {
        try {
            if (registry.getConfigSystemRegistry().getRegistryContext() != null &&
                registry.getConfigSystemRegistry().getRegistryContext().getDataSource() != null) {
                registryDriver = registry.getConfigSystemRegistry().getRegistryContext()
                        .getDataSource().getConnection().getMetaData().getDriverName();
            }

        } catch (Exception e) {
            String msg = "Cannot get registry driver";
            log.error(msg, e);
        }
    }

    public static String getRegistryDatabaseDriver() {
        return registryDriver;
    }

    public static String getUserManagerDriver() {
        return usermanagerDriver;
    }

    
    protected void setRegistryService(RegistryService registryService) {
        this.registryService = registryService;
    }

    protected void unsetRegistryService(RegistryService registryService) {
        this.registryService = null;
        registryDriver = null;
    }

    protected void setRealmService(RealmService realmService) {
        this.realmService = realmService;
    }

    protected void unsetRealmService(RealmService realmService) {
        this.realmService = null;
        usermanagerDriver = null;
    }

    public static RealmService getRealmService() {
        return realmService;
    }

    protected void setConfigurationContextService(ConfigurationContextService contextService) {
        this.configContext = contextService.getServerConfigContext();
    }

    protected void unsetConfigurationContextService(ConfigurationContextService contextService) {
        AxisConfiguration axisConf = configContext.getAxisConfiguration();
        AxisModule statModule = axisConf.getModule(ServerAdminServiceComponent.SERVER_ADMIN_MODULE_NAME);
        if (statModule != null) {
            try {
                axisConf.disengageModule(statModule);
            } catch (AxisFault axisFault) {
                log.error("Failed disengage module: " + ServerAdminServiceComponent.SERVER_ADMIN_MODULE_NAME);
            }
        }
        this.configContext = null;
    }

    protected void setServerConfiguration(ServerConfiguration serverConfiguration) {
        serverConfig = serverConfiguration;
    }

    protected void unsetServerConfiguration(ServerConfiguration serverConfiguration) {
        serverConfig = null;
    }

    public static RegistryService getRegistryService() {
        return registryService;
    }

    public static ServerConfiguration getServerConfig() {
        return serverConfig;
    }
}
