/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.server;

import org.apache.catalina.Context;
import org.apache.catalina.Host;
import org.apache.catalina.startup.Embedded;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.utils.CarbonUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 *
 */
@SuppressWarnings({"ConstantConditions", "InfiniteLoopStatement"})
public class TomcatRepositoryListener implements Runnable {

    private static Log log = LogFactory.getLog(TomcatRepositoryListener.class);

    private Map<String, WebContext> repoMap = new HashMap<String, WebContext>();
    private Embedded embedded;
    private Host host;
    private File repo;

    public TomcatRepositoryListener(Embedded embedded,
                                    Host host,
                                    String repo) throws Exception {
        this.embedded = embedded;
        this.host = host;
        this.repo = new File(repo);
        if (!this.repo.isDirectory()) {
            this.repo = null;
            throw new Exception(repo + " is not a directory");
        }
    }

    public void run() {
//        ServerConfiguration serverConfig = CarbonUtils.getServerConfiguration();

        String carbonWebContextRoot = "/";
        try {
            File carbonXml = new File(CarbonUtils.getCarbonXml());
            Document doc =
                    DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(carbonXml);
            doc.getDocumentElement().normalize();
            Element rootEle = doc.getDocumentElement();
            carbonWebContextRoot =
                    rootEle.getElementsByTagName("WebContextRoot").item(0).getTextContent();
        } catch (Exception e) {
            log.fatal("Cannot parse carbon.xml file to find the Carbon webapp WebContextRoot entry",
                      e);
            return;
        }

        while (true) {
            try {
                File[] files = repo.listFiles();
                Collection<String> webapps = new ArrayList<String>();

                // Deploy new files
                if (files != null) {
                    for (File file : files) {
                        String filename = file.getName();
                        webapps.add(filename);
                        if (!repoMap.containsKey(filename)) { // Hot deployment
                            if (file.isDirectory()) {
                                boolean isCarbonWebapp =
                                        new File(file.getAbsolutePath() + File.separator +
                                                 "wso2carbon-version.txt").exists();
                                Context context;
                                if (isCarbonWebapp && carbonWebContextRoot != null){
                                    context = embedded.createContext(carbonWebContextRoot,
                                                                     file.getAbsolutePath());
                                    host.addChild(context);
                                } else {
                                    if (filename.equals("ROOT")) {
                                        context = embedded.createContext(carbonWebContextRoot,
                                                                         file.getAbsolutePath());
                                        host.addChild(context);
                                    } else {
                                        context =
                                                embedded.createContext("/" + filename, file.getAbsolutePath());
                                        host.addChild(context);
                                    }
                                }

                                if (log.isDebugEnabled()) {
                                    log.debug("Deployed Web application: " + filename);
                                }

                                repoMap.put(filename, new WebContext(file.lastModified(), context));
                            } else if (filename.endsWith(".war")) { // Handle WAR
                                String warContext = filename.substring(0, filename.indexOf(".war"));
                                Context context =
                                        embedded.createContext("/" + warContext,
                                                               file.getAbsolutePath());
                                context.setReloadable(false);
                                host.addChild(context);
                            }
                        }
                    }

                    // Undeploy old files
                    Collection<String> toRemove = new ArrayList<String>();
                    for (String filename : repoMap.keySet()) {
                        if (!webapps.contains(filename)) {
                            Context context = (repoMap.get(filename)).getContext();
                            if (context != null) {
                                host.removeChild(context);
                                toRemove.add(filename);
                                if (log.isDebugEnabled()) {
                                    log.debug("Undeployed Web application " + filename);
                                }
                            }
                        }
                    }
                    for (String aToRemove : toRemove) {
                        repoMap.remove(aToRemove);
                    }
                }

                // Poll the repo every few seconds
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                    log.error("TomcatRepositoryListener thread was interrupted", e);
                }
            } catch (Throwable e) {
                log.error("Error occurred while polling tomcat webapps repository", e);
            }
        }
    }

    private static class WebContext {
        private long timestamp;
        private Context context;

        public WebContext(long timestamp, Context context) {
            this.timestamp = timestamp;
            this.context = context;
        }

        public long getTimestamp() {
            return timestamp;
        }

        public Context getContext() {
            return context;
        }

        public void setTimestamp(long timestamp) {
            this.timestamp = timestamp;
        }
    }
}
