/*                                                                             
 * Copyright 2004,2005 The Apache Software Foundation.                         
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.carbon.server.transports;

import java.util.List;

/**
 * All standalone transports which need to be started up when the server is started up, need to
 * implement this interface
 */
public interface Transport {

    /**
     * Initialize the transport
     *
     * @param parameters The list of TransportParameters
     * @throws Exception If an error occurred during initializing
     */
    void init(List<TransportParameter> parameters) throws Exception;

    /**
     * Start the transport
     *
     * @throws Exception If an error occurred during starting
     */
    void start() throws Exception;

    /**
     * Stop the transport
     *
     * @throws Exception If an error occurred while stopping
     */
    void stop() throws Exception;

    /**
     * Get the tranport port. If port is not applicable for a particular transport, returns -1.
     *
     * @return The transport port. If port is not applicable for a particular transport, returns -1.
     */
    int getPort();

    /**
     * Get the tranport proxy port. If port is not applicable for a particular transport, returns -1.
     *
     * @return The transport proxy port. If port is not applicable for a particular transport, returns -1.
     */
    int getProxyPort();
}
