package org.wso2.carbon.ui;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.ServiceReference;
import org.wso2.carbon.ui.action.ActionHelper;
import org.wso2.carbon.ui.deployment.beans.CarbonUIDefinitions;

public class TilesJspServlet extends JspServlet {
    private static final long serialVersionUID = 1L;
    private static Log log = LogFactory.getLog(TilesJspServlet.class);

    public TilesJspServlet(Bundle bundle, String bundleResourcePath, BundleResourcePathRegistry resourcePathRegistry) {
        super(bundle, bundleResourcePath, resourcePathRegistry);
    }

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        String actionUrl = request.getRequestURI();

        //This is the layout page defined in
        //"/org.wso2.carbon.component/src/main/resources/web/WEB-INF/tiles/main_defs.xml"
        //Need to serve http requests other than to tiles body page,
        //using the normal OSGi way

        //retrieve urls that should be by-passed from tiles servlet
        String resourceURI = actionUrl.replaceFirst(request.getContextPath() + "/", "../");

        HashMap<String, String> urlsToBeByPassed = new HashMap<String, String>();
        if (bundle != null) {
            ServiceReference reference = bundle.getBundleContext()
                    .getServiceReference(CarbonUIDefinitions.class.getName());
            CarbonUIDefinitions carbonUIDefinitions = null;
            if (reference != null) {
                carbonUIDefinitions =
                        (CarbonUIDefinitions) bundle.getBundleContext().getService(reference);
                if (carbonUIDefinitions != null) {
                    urlsToBeByPassed = carbonUIDefinitions.getSkipTilesUrls();
                }
            }
        }

        //if the current uri is marked to be by-passed, let it pass through
        if (!urlsToBeByPassed.isEmpty()) {
            if (urlsToBeByPassed.containsKey(resourceURI)) {
                super.service(request, response);
                return;
            }
        }


        if ((actionUrl.lastIndexOf("/admin/layout/template.jsp") > -1)
                || actionUrl.lastIndexOf("ajaxprocessor.jsp") > -1
                || actionUrl.indexOf("gadgets/js") > -1) {
            super.service(request, response);
        } else if (actionUrl.startsWith("/carbon/registry/web/resources/foo/bar")) {
            //TODO : consider the renamed ROOT war scenario
            String resourcePath = actionUrl.replaceFirst("/carbon/registry/web/", "");
            String pathToRegistry = "path=" + resourcePath;
            if (log.isTraceEnabled()) {
                log.trace("Forwarding to registry : " + pathToRegistry);
            }
            RequestDispatcher dispatcher =
                    request.getRequestDispatcher("../registry/registry-web.jsp?" + pathToRegistry);
            dispatcher.forward(request, response);
        } else {
            try {
                if (log.isDebugEnabled()) {
                    log.debug("request.getContextPath() : " + request.getContextPath());
                    log.debug("actionUrl : " + actionUrl);
                }
                String newPath = actionUrl.replaceFirst(request.getContextPath(), "");

                //todo: Insert filter work here

                ActionHelper.render(newPath, request, response);
            } catch (Exception e) {
                log.fatal("Fatal error occurred while rendering UI using Tiles.", e);
            }
        }
    }
}
