package org.wso2.carbon.user.core.common;

import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreManager;
import org.wso2.carbon.user.core.claim.Claim;
import org.wso2.carbon.user.core.claim.ClaimManager;
import org.wso2.carbon.user.core.claim.ClaimMapping;
import org.wso2.carbon.user.core.config.RealmConfiguration;
import org.wso2.carbon.user.core.hybrid.HybridRoleManager;
import org.wso2.carbon.user.core.profile.ProfileConfiguration;
import org.wso2.carbon.user.core.profile.ProfileConfigurationManager;

import javax.sql.DataSource;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class AbstractUserStoreManager implements UserStoreManager {

    protected DataSource dataSource = null;
    protected RealmConfiguration realmConfig = null;
    protected ClaimManager claimManager = null;
    protected ProfileConfigurationManager profileManager = null;
    protected UserRealm userRealm = null;
    protected HybridRoleManager hybridRoleManager = null;

    public String getUserClaimValue(String userName, String claim, String profileName)
            throws UserStoreException {
        String profileConfiguration = null;

        if (profileConfiguration == null) {
            profileConfiguration = UserCoreConstants.DEFAULT_PROFILE_CONFIGURATION;
        }

        String property = claimManager.getAttributeName(claim);

        ProfileConfiguration profileConfig = profileManager.getProfileConfig(profileConfiguration);
        if (profileConfig != null) {
            List<String> hidden = getMappingAttributeList(profileConfig.getHiddenClaims());
            if (hidden.contains(property)) {
                return null;
            }
        }

        String value = this.getUserPropertyValues(userName, new String[] { property }, profileName)
                .get(property);
        if ((value == null || value.trim().length() == 0)
                && !UserCoreConstants.DEFAULT_PROFILE.equals(profileName) && profileConfig != null) {
            List<String> overridden = getMappingAttributeList(profileConfig.getOverriddenClaims());
            if (!overridden.contains(property)) {
                value = this.getUserPropertyValues(userName, new String[] { property },
                        UserCoreConstants.DEFAULT_PROFILE).get(property);
            }
        }
        return value;

    }

    public Claim[] getUserClaimValues(String userName, String profileName)
            throws UserStoreException {
        if (profileName == null) {
            profileName = UserCoreConstants.DEFAULT_PROFILE;
        }

        String[] claims = claimManager.getAllClaimUris();
        Map<String, String> values = this.getUserClaimValues(userName, claims, profileName);
        Claim[] finalValues = new Claim[values.size()];
        int i = 0;
        for (Iterator<Map.Entry<String, String>> ite = values.entrySet().iterator(); ite.hasNext();) {
            Map.Entry<String, String> entry = ite.next();
            Claim claim = new Claim();
            claim.setValue(entry.getValue());
            claim.setClaimUri(entry.getKey());
            String displayTag = claimManager.getClaim(entry.getKey()).getDisplayTag();
            claim.setDisplayTag(displayTag);
            finalValues[i] = claim;
            i++;
        }
        return finalValues;
    }

    public Map<String, String> getUserClaimValues(String userName, String[] claims,
            String profileName) throws UserStoreException {

        if (profileName == null) {
            profileName = UserCoreConstants.DEFAULT_PROFILE;
        }

        String profileConfigurationName = null;
        boolean isPofileCnfigurationRequested = false;

        Set<String> propertySet = new HashSet<String>();
        for (String claim : claims) {
            ClaimMapping mapping = null;
            mapping = claimManager.getClaimMapping(claim);
            // There can be cases some claim values being requested for claims
            // we don't have.
            if (mapping != null) {
                propertySet.add(mapping.getMappedAttribute());
            }

            if (UserCoreConstants.PROFILE_CONFIGURATION.equals(claim)) {
                isPofileCnfigurationRequested = true;
            }
        }

        propertySet.add(UserCoreConstants.PROFILE_CONFIGURATION);

        String[] properties = propertySet.toArray(new String[propertySet.size()]);
        Map<String, String> uerProperties = this.getUserPropertyValues(userName, properties,
                profileName);
        profileConfigurationName = uerProperties.get(UserCoreConstants.PROFILE_CONFIGURATION);
        if (profileConfigurationName == null) {
            profileConfigurationName = UserCoreConstants.DEFAULT_PROFILE_CONFIGURATION;
        }

        ProfileConfiguration profileConfig = profileManager
                .getProfileConfig(profileConfigurationName);

        List<String> hidden = null;
        if (profileConfig != null) {
            hidden = getMappingAttributeList(profileConfig.getHiddenClaims());
        }

        List<String> overridden = null;
        if (profileConfig != null) {
            overridden = getMappingAttributeList(profileConfig.getOverriddenClaims());
        }

        List<String> getAgain = new ArrayList<String>();
        Map<String, String> finalValues = new HashMap<String, String>();

        if (isPofileCnfigurationRequested) {
            finalValues.put(UserCoreConstants.PROFILE_CONFIGURATION, profileConfigurationName);
        }

        for (String claim : claims) {
            ClaimMapping mapping = null;
            mapping = claimManager.getClaimMapping(claim);
            if (mapping != null) {
                String property = mapping.getMappedAttribute();
                String value = uerProperties.get(property);
                if (hidden != null && hidden.contains(property)) {
                    continue;
                }

                if (profileName.equals(UserCoreConstants.DEFAULT_PROFILE)) {
                    // Check whether we have a value for the requested attribute
                    if (value != null && value.trim().length() > 0) {
                        finalValues.put(claim, value);
                    }
                } else if (profileConfig != null && (value == null || value.equals(""))) {
                    if (overridden != null && overridden.contains(property)) {
                        finalValues.put(claim, value);
                    } else {
                        getAgain.add(claim);
                    }
                } else {
                    if (value != null && value.trim().length() > 0) {
                        finalValues.put(claim, value);
                    }
                }
            }
        }

        if (getAgain.size() > 0) {
            // oh the beautiful recursion
            Map<String, String> mapClaimValues = this.getUserClaimValues(userName,
                    (String[]) getAgain.toArray(new String[getAgain.size()]),
                    UserCoreConstants.DEFAULT_PROFILE);

            Iterator<Map.Entry<String, String>> ite3 = mapClaimValues.entrySet().iterator();
            while (ite3.hasNext()) {
                Map.Entry<String, String> entry = ite3.next();
                if (entry.getValue() != null) {
                    finalValues.put(entry.getKey(), entry.getValue());
                }
            }
        }

        return finalValues;
    }

    protected List<String> getMappingAttributeList(List<String> claimList)
            throws UserStoreException {
        ArrayList<String> attributeList = null;
        Iterator<String> claimIter = null;

        attributeList = new ArrayList<String>();
        if (claimList == null) {
            return attributeList;
        }
        claimIter = claimList.iterator();
        while (claimIter.hasNext()) {
            attributeList.add(claimManager.getAttributeName(claimIter.next()));
        }
        return attributeList;
    }

//    public String[] getHybridRoles() throws UserStoreException {
//        return hybridRoleManager.getHybridRoles();
//    }
//
//    public String[] getUserListOfHybridRole(String roleName) throws UserStoreException {
//        return hybridRoleManager.getUserListOfHybridRole(roleName);
//    }
//
//    public void updateUserListOfHybridRole(String roleName, String[] deletedUsers, String[] addUsers)
//            throws UserStoreException {
//        hybridRoleManager.updateUserListOfHybridRole(roleName, deletedUsers, addUsers);
//    }
//
//    public String[] getHybridRoleListOfUser(String user) throws UserStoreException {
//        return hybridRoleManager.getHybridRoleListOfUser(user);
//    }
//
//    public void updateHybridRolesOfUser(String user, String[] deletedRoles, String[] addRoles)
//            throws UserStoreException {
//        hybridRoleManager.updateHybridRolesOfUser(user, deletedRoles, addRoles);
//    }

    /**
     * This method is used by the support system to read properties
     */
    public abstract Map<String, String> getUserPropertyValues(String userName,
            String[] propertyNames, String profileName) throws UserStoreException;

    protected boolean checkUserNamePasswordValid(String userName, Object credential)
            throws UserStoreException {
        if (credential == null) {
            return false;
        }

        if (userName == null || CarbonConstants.REGISTRY_SYSTEM_USERNAME.equals(userName)) {
            return false;
        }

        if (!(credential instanceof String)) {
            throw new UserStoreException("Can handle only string type credentials");
        }

        userName = userName.trim();
        String password = ((String) credential).trim();

        if (userName.length() < 1 || password.length() < 1) {
            return false;
        }

        String regEx = realmConfig.getUserStoreProperty(UserCoreConstants.RealmConfig.PROPERTY_JAVA_REG_EX);
        if (regEx != null) {
            Pattern p = Pattern.compile(regEx);
            Matcher m = p.matcher(password);
            if (!m.matches()) {
                return false;
            }
        }
        
        return true;
    }
}
