/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.user.core.common;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.Date;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.claim.ClaimMapping;
import org.wso2.carbon.user.core.claim.builder.ClaimBuilder;
import org.wso2.carbon.user.core.claim.builder.ClaimBuilderException;
import org.wso2.carbon.user.core.claim.dao.ClaimDAO;
import org.wso2.carbon.user.core.config.RealmConfigXMLProcessor;
import org.wso2.carbon.user.core.config.RealmConfiguration;
import org.wso2.carbon.user.core.jdbc.JDBCRealmConstants;
import org.wso2.carbon.user.core.profile.ProfileConfiguration;
import org.wso2.carbon.user.core.profile.builder.ProfileBuilderException;
import org.wso2.carbon.user.core.profile.builder.ProfileConfigurationBuilder;
import org.wso2.carbon.user.core.profile.dao.ProfileConfigDAO;
import org.wso2.carbon.user.core.service.RealmService;
import org.wso2.carbon.user.core.tenant.TenantManager;
import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;

public class DefaultRealmService implements RealmService {
    Map<Integer, UserRealm> userRealmMap = new HashMap<Integer, UserRealm>();
    BundleContext bc;
    RealmConfiguration bootstrapRealmConfig;
    DataSource dataSource;
    OMElement parentElement;
    TenantManager tenantManager;
    UserRealm bootstrapRealm;
    private static final String TENANT_CONFIG_XML = "tenant-config.xml";
    private static final Log log = LogFactory.getLog(DefaultRealmService.class);

    public DefaultRealmService(BundleContext bc) throws Exception {

        this.parentElement = getConfigurationElement();
        // TODO this should go away once the system permissions are gone
        OMElement realmElement = parentElement.getFirstChildWithName(new QName(
                UserCoreConstants.RealmConfig.LOCAL_NAME_REALM));
        RealmConfigXMLProcessor rmProcessor = new RealmConfigXMLProcessor();
        this.bootstrapRealmConfig = rmProcessor.buildRealmConfiguration(realmElement);
        this.dataSource = getDataSource(bootstrapRealmConfig);
        initializeDatabase(dataSource);

        this.tenantManager = this.initializeTenantManger(this.getTenantConfigurationElement(bc),
                this.dataSource);
        this.tenantManager.setBundleContext(bc);

        // initializing the bootstrapRealm
        this.bc = bc;
        bootstrapRealm = initializeRealm(bootstrapRealmConfig, dataSource, 0);
        Dictionary<String, String> dictionary = new Hashtable<String, String>();
        dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.DELEGATING_REALM);
        if (bc != null) {
            // note that in a case of we don't run this in an OSGI envrionment
            // like checkin-client,
            // we need to avoid the registration of the service
            bc.registerService(UserRealm.class.getName(), bootstrapRealm, dictionary);
        }
    }

    public UserRealm getUserRealm(RealmConfiguration tenantRealmConfig)
			throws UserStoreException {
		UserRealm userRealm = null;
		int tenantId = tenantRealmConfig.getTenantId();
		userRealm = userRealmMap.get(new Integer(tenantId));
		if (userRealm == null && tenantId == 0) {
			userRealm = bootstrapRealm;
		}

		if (userRealm == null) {
			userRealm = initializeRealm(tenantRealmConfig, dataSource, tenantId);
			userRealmMap.put(new Integer(tenantId), userRealm);
		} else {
			Date persistedTime = userRealm.getRealmConfiguration()
					.getPersistedTimestamp();
			if (persistedTime != null
					&& !persistedTime.equals(tenantRealmConfig
							.getPersistedTimestamp())) {
				// this is an update
				userRealm = initializeRealm(tenantRealmConfig, dataSource,
						tenantId);
				userRealmMap.put(new Integer(tenantId), userRealm);
			}
		}
		return userRealm;
	}
    
    public void setUserRealm(int id, UserRealm realm) throws UserStoreException {
        userRealmMap.put(new Integer(id), realm);
    }

    public UserRealm initializeRealm(RealmConfiguration realmConfig, DataSource dataSource,
            int tenantId) throws UserStoreException {

        ClaimBuilder.setBundleContext(bc);
        ProfileConfigurationBuilder.setBundleContext(bc);
        
        UserRealm userRealm = new DefaultRealm();
        
        Map<String, ClaimMapping> claimMappings;
        Map<String, ProfileConfiguration> profileConfigs;
        ClaimDAO claimDAO = new ClaimDAO(dataSource, tenantId);
        ProfileConfigDAO profileDAO = new ProfileConfigDAO(dataSource, tenantId);
        int count = claimDAO.getDialectCount();
        ClaimBuilder claimBuilder = new ClaimBuilder(tenantId);
        ProfileConfigurationBuilder profileBilder = new ProfileConfigurationBuilder(tenantId);

        if (count == 0) {
            try {
                claimMappings = claimBuilder.buildClaimMappingsFromConfigFile();
            } catch (ClaimBuilderException e) {
                String msg = "Error in building claims.";
                log.error(msg);
                throw new UserStoreException(msg, e);
            }
            claimDAO.addCliamMappings(claimMappings.values().toArray(
                    new ClaimMapping[claimMappings.size()]));
            try {
                profileConfigs = profileBilder.buildProfileConfigurationFromConfigFile();
            } catch (ProfileBuilderException e) {
                String msg = "Error in building the profile.";
                log.error(msg);
                throw new UserStoreException(msg, e);
            }
            profileDAO.addProfileConfig(profileConfigs.values().toArray(
                    new ProfileConfiguration[profileConfigs.size()]));
        } else {
            try {
                claimMappings = claimBuilder.buildClaimMappingsFromDatabase(dataSource,
                        UserCoreConstants.INTERNAL_USERSTORE);
            } catch (ClaimBuilderException e) {
                String msg = "Error in building claims.";
                log.error(msg);
                throw new UserStoreException(msg, e);
            }
            try {
                profileConfigs = profileBilder.buildProfileConfigurationFromDatabase(dataSource,
                        UserCoreConstants.INTERNAL_USERSTORE);
            } catch (ProfileBuilderException e) {
                String msg = "Error in building the profile.";
                log.error(msg);
                throw new UserStoreException(msg, e);
            }
        }

        userRealm.init(realmConfig, claimMappings, profileConfigs, tenantId);

        return userRealm;
    }

    // TODO : Move this into RealmConfigXMLProcessor
    private OMElement getConfigurationElement() {
        try {
            String userMgt = CarbonUtils.getUserMgtXMLPath();
            InputStream inStream = null;
            if (userMgt != null) {
                File userMgtXml = new File(userMgt);
                if (!userMgtXml.exists()) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found.";
                    throw new FileNotFoundException(msg);
                }
                inStream = new FileInputStream(userMgtXml);
            } else {
                inStream = this.getClass().getClassLoader().getResourceAsStream(
                        "repository/conf/user-mgt.xml");
                if (inStream == null) {
                    String msg = "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found. Please set the carbon.home";
                    throw new FileNotFoundException(msg);
                }
            }

            StAXOMBuilder builder = new StAXOMBuilder(inStream);
            return builder.getDocumentElement();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }

        return null;
    }

    private void initializeDatabase(DataSource ds) throws Exception {

        String value = System.getProperty("setup");
        if (value != null) {
            DatabaseCreator databaseCreator = new DatabaseCreator(ds);
            try {
                databaseCreator.createRegistryDatabase();
            } catch (Exception e) {
                String msg = "Error in creating the database";
                throw new Exception(msg, e);
            }
        }
    }

    @SuppressWarnings("unchecked")
    private TenantManager initializeTenantManger(OMElement configElement, DataSource dataSource)
            throws Exception {
        TenantManager tenantManager = null;
        String className = configElement.getAttribute(new QName("class")).getAttributeValue();
        Class clazz = Class.forName(className);
        Map<String, Object> properties = new HashMap<String, Object>();
        properties.put(UserCoreConstants.DATA_SOURCE, dataSource);
        Constructor constructor = clazz.getConstructor(OMElement.class, Map.class);
        Object newObject = constructor.newInstance(configElement, properties);
        tenantManager = (TenantManager) newObject;
        return tenantManager;
    }

    private DataSource getDataSource(RealmConfiguration realmConfig) {
        BasicDataSource ds = new BasicDataSource();
        ds.setDriverClassName(realmConfig.getRealmProperty(JDBCRealmConstants.DRIVER_NAME));
        ds.setUrl(realmConfig.getRealmProperty(JDBCRealmConstants.URL));
        ds.setUsername(realmConfig.getRealmProperty(JDBCRealmConstants.USER_NAME));
        ds.setPassword(realmConfig.getRealmProperty(JDBCRealmConstants.PASSWORD));
        ds.setMaxActive(Integer.parseInt(realmConfig
                .getRealmProperty(JDBCRealmConstants.MAX_ACTIVE)));
        ds.setMaxIdle(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MIN_IDLE)));
        ds.setMaxWait(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MAX_WAIT)));
        ds.setMinIdle(Integer.parseInt(realmConfig.getRealmProperty(JDBCRealmConstants.MIN_IDLE)));
        return ds;
    }

    public RealmConfiguration getBootstrapRealmConfiguration() {
        return bootstrapRealmConfig;
    }

    public UserRealm getBootstrapRealm() throws UserStoreException {
        return bootstrapRealm;
    }

    public TenantManager getTenantManager() {
        return this.tenantManager;
    }

    private OMElement getTenantConfigurationElement(BundleContext bundleContext) throws Exception {
        InputStream inStream = null;
        File claimConfigXml = new File(CarbonUtils.getCarbonConfigDirPath(), TENANT_CONFIG_XML);
        if (claimConfigXml.exists()) {
            inStream = new FileInputStream(claimConfigXml);
        }
        if (inStream == null) {
            if (bundleContext != null) {
                inStream = bundleContext.getBundle().getResource(TENANT_CONFIG_XML).openStream();
            } else {
                inStream = this.getClass().getClassLoader().getResource(TENANT_CONFIG_XML)
                        .openStream();
            }
        }

        if (inStream == null) {
            String message = "Tenant configuration not found.";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new FileNotFoundException(message);
        }

        StAXOMBuilder builder = new StAXOMBuilder(inStream);
        OMElement documentElement = builder.getDocumentElement();

        return documentElement;
    }
}
