/*
 *  Copyright (c) 2005-2009, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.user.core.internal;

import org.wso2.carbon.user.core.listener.AuthorizationManagerListener;
import org.wso2.carbon.user.core.listener.UserStoreManagerListener;

import java.util.Map;
import java.util.Collection;
import java.util.TreeMap;

/**
 * @scr.component name="org.wso2.carbon.user.core.listener" immediate="true"
 * @scr.reference name="authorization.manager.listener.service"
 *                interface="org.wso2.carbon.user.core.listener.AuthorizationManagerListener"
 *                cardinality="0..n" policy="dynamic"
 *                bind="setAuthorizationManagerListenerService"
 *                unbind="unsetAuthorizationManagerListenerService"
 * @scr.reference name="user.store.manager.listener.service"
 *                interface="org.wso2.carbon.user.core.listener.UserStoreManagerListener"
 *                cardinality="0..n" policy="dynamic"
 *                bind="setUserStoreManagerListenerService"
 *                unbind="unsetUserStoreManagerListenerService"
 */
public class UMListenerServiceComponent {

    private static Map<Integer, AuthorizationManagerListener> authorizationManagerListeners;
    private static Map<Integer, UserStoreManagerListener> userStoreManagerListeners;
    private static Collection<AuthorizationManagerListener> authorizationManagerListenerCollection;
    private static Collection<UserStoreManagerListener> userStoreManagerListenerCollection;

    protected void setAuthorizationManagerListenerService(
            AuthorizationManagerListener authorizationManagerListenerService) {
        authorizationManagerListenerCollection = null;
        if (authorizationManagerListeners == null) {
            authorizationManagerListeners =
                    new TreeMap<Integer, AuthorizationManagerListener>();
        }
        authorizationManagerListeners.put(authorizationManagerListenerService.getExecutionOrderId(),
                authorizationManagerListenerService);
    }

    protected void unsetAuthorizationManagerListenerService(
            AuthorizationManagerListener authorizationManagerListenerService) {
        if (authorizationManagerListenerService != null
                && authorizationManagerListeners != null) {
            authorizationManagerListeners.remove(
                    authorizationManagerListenerService.getExecutionOrderId());
            authorizationManagerListenerCollection = null;
        }
    }

    protected void setUserStoreManagerListenerService(
            UserStoreManagerListener userStoreManagerListenerService) {
        userStoreManagerListenerCollection = null;
        if (userStoreManagerListeners == null) {
            userStoreManagerListeners =
                    new TreeMap<Integer, UserStoreManagerListener>();
        }
        userStoreManagerListeners.put(userStoreManagerListenerService.getExecutionOrderId(),
                userStoreManagerListenerService);
    }

    protected void unsetUserStoreManagerListenerService(
            UserStoreManagerListener userStoreManagerListenerService) {
        if (userStoreManagerListenerService != null &&
                userStoreManagerListeners != null) {
            userStoreManagerListeners.remove(userStoreManagerListenerService.getExecutionOrderId());
            userStoreManagerListenerCollection = null;
        }
    }

    public static Collection<AuthorizationManagerListener> getAuthorizationManagerListeners() {
        if (authorizationManagerListeners == null) {
            authorizationManagerListeners = new TreeMap<Integer, AuthorizationManagerListener>();
        }
        if (authorizationManagerListenerCollection == null) {
            authorizationManagerListenerCollection =
                    authorizationManagerListeners.values();
        }
        return authorizationManagerListenerCollection;
    }

    public static Collection<UserStoreManagerListener> getUserStoreManagerListeners() {
        if (userStoreManagerListeners == null) {
            userStoreManagerListeners = new TreeMap<Integer, UserStoreManagerListener>();
        }
        if (userStoreManagerListenerCollection == null) {
            userStoreManagerListenerCollection =
                    userStoreManagerListeners.values();
        }
        return userStoreManagerListenerCollection;
    }
}
