package org.wso2.carbon.user.core.profile.builder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleContext;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.profile.ProfileConfiguration;
import org.wso2.carbon.user.core.profile.dao.ProfileConfigDAO;
import org.wso2.carbon.utils.ServerConstants;
import org.wso2.carbon.utils.CarbonUtils;

public class ProfileConfigurationBuilder {

    private static Log log = LogFactory.getLog(ProfileConfigurationBuilder.class);

    private static final String PROFILE_CONFIG = "profile-config.xml";
    
    public static final String LOCAL_NAME_USER_PROFILES = "UserProfiles";

    public static final String ATTR_PROFILE_NAME = "profileName";
    public static final String ATTR_PROFILE_CONFIG_NAME = "configName";

    public static final String LOCAL_NAME_PROFILE_CONFIG = "ProfileConfiguration";
    public static final String LOCAL_NAME_CLAIM = "Claim";
    public static final String LOCAL_NAME_CLAIM_URI = "ClaimURI";
    public static final String LOCAL_NAME_ATTR_ID = "AttributeID";
    public static final String LOCAL_NAME_PROFILES = "Profiles";
    public static final String LOCAL_NAME_PROFILE = "Profile";
    public static final String LOCAL_NAME_CLAIM_BEHAVIOR = "ClaimBehavior";
    public static final String ATTR_DIALECT_URI = "dialectURI";
    

    private static BundleContext bundleContext;
    private InputStream inStream = null;
    int tenantId;

    public ProfileConfigurationBuilder(int tenantId) {
        this.tenantId = tenantId;
    }

    public static void setBundleContext(BundleContext bundleContext) {
        ProfileConfigurationBuilder.bundleContext = bundleContext;
    }

    public Map<String, ProfileConfiguration> buildProfileConfigurationFromDatabase(DataSource ds, String realmName)
            throws ProfileBuilderException {
        try {
            ProfileConfigDAO profileDAO = new ProfileConfigDAO(ds, tenantId);
            Map<String, ProfileConfiguration> profileConfigs = profileDAO.loadProfileConfigs();
            return profileConfigs;
        } catch (UserStoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            throw new ProfileBuilderException();
        }
    }

    /**
     * 
     * @return
     * @throws ProfileBuilderException
     */
    public Map<String, ProfileConfiguration> buildProfileConfigurationFromConfigFile()
            throws ProfileBuilderException {
        OMElement element = null;
        String message = null;
        Iterator<OMElement> configIterator = null;
        Map<String, ProfileConfiguration> profileConfigs = null;

        try {
            element = getRootElement();
        } catch (Exception e) {
            message = "Error while reading profile configuration";
            log.error(message, e);
            throw new ProfileBuilderException(message, e);
        }

        configIterator = element
                .getChildrenWithLocalName(LOCAL_NAME_PROFILE_CONFIG);

        profileConfigs = new HashMap<String, ProfileConfiguration>();

        while (configIterator.hasNext()) {
            OMElement profileElem = configIterator.next();
            String dialectName = null;
            String profileConfigName = null;
            Iterator<OMElement> profileIter = null;
            List<String> hidden = null;
            List<String> overridden = null;
            List<String> inherited = null;
            ProfileConfiguration profileConfiguration = null;

            profileConfigName = profileElem.getAttribute(
                    new QName(ATTR_PROFILE_CONFIG_NAME)).getAttributeValue();
            dialectName = profileElem.getAttribute(new QName(ATTR_DIALECT_URI))
                    .getAttributeValue();
            profileIter = profileElem.getChildrenWithLocalName(LOCAL_NAME_CLAIM);

            hidden = new ArrayList<String>();
            overridden = new ArrayList<String>();
            inherited = new ArrayList<String>();

            while (profileIter.hasNext()) {
                OMElement claimElem = null;
                String claimURI = null;
                String behavior = null;

                claimElem = profileIter.next();
                claimURI = claimElem.getFirstChildWithName(
                        new QName(LOCAL_NAME_CLAIM_URI)).getText();
                behavior = claimElem.getFirstChildWithName(
                        new QName(LOCAL_NAME_CLAIM_BEHAVIOR)).getText();
                if (behavior.equals(UserCoreConstants.CLAIM_HIDDEN)) {
                    hidden.add(claimURI);
                } else if (behavior.equals(UserCoreConstants.CLAIM_OVERRIDEN)) {
                    overridden.add(claimURI);
                } else {
                    inherited.add(claimURI);
                }
            }
            profileConfiguration = new ProfileConfiguration(profileConfigName, hidden, overridden,
                    inherited);
            profileConfiguration.setDialectName(dialectName);
            profileConfigs.put(profileConfigName, profileConfiguration);

        }
        
        try {
            if (inStream != null) {
                inStream.close();
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new ProfileBuilderException(e.getMessage(), e);
        }
        
        
        return profileConfigs;
    }

    /**
     * 
     * @return
     * @throws XMLStreamException
     * @throws IOException
     * @throws ProfileBuilderException
     */
    private OMElement getRootElement() throws XMLStreamException, IOException,
            ProfileBuilderException {
        String carbonHome = null;
        StAXOMBuilder builder = null;

        carbonHome = System.getProperty(ServerConstants.CARBON_HOME);

        if (carbonHome != null) {
            File profileConfigXml = new File(CarbonUtils.getCarbonConfigDirPath(), PROFILE_CONFIG);
            if (profileConfigXml.exists()) {
                inStream = new FileInputStream(profileConfigXml);
            }
        }

        if (inStream == null) {
            if (bundleContext != null) {
                inStream = bundleContext.getBundle().getResource(PROFILE_CONFIG).openStream();
            } else {
                inStream = this.getClass().getClassLoader().getResource(PROFILE_CONFIG).openStream();
            }
        }
        
        if (inStream == null) {
            String message = "Profile configuration not found.";
            if (log.isDebugEnabled()) {
                log.debug(message);
            }
            throw new FileNotFoundException(message);
        }

        builder = new StAXOMBuilder(inStream);
        OMElement documentElement = builder.getDocumentElement();
              
        return documentElement;
    }

}
