/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.user.core.service;

import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.config.RealmConfiguration;
import org.wso2.carbon.user.core.tenant.TenantManager;

public interface RealmService {
    /**
     * Get a user realm from a given configuration
     * @param tenantRealmConfig
     * @return UserRealm
     * @throws UserStoreException
     */
    public UserRealm getUserRealm(RealmConfiguration tenantRealmConfig) throws UserStoreException;

    /**
     * Get the boot strap realm configuration, this is always build from conf/user-mgt.xml
     * @return tge realm configuration
     */
    public RealmConfiguration getBootstrapRealmConfiguration();

    /**
     * Get the bootstrap realm 
     * @return
     * @throws UserStoreException
     */
    public UserRealm getBootstrapRealm() throws UserStoreException;
    
    /**
     * 
     * @param id
     * @param realm
     * @throws UserStoreException
     */
    public void setUserRealm(int id, UserRealm realm) throws UserStoreException;

    /**
     * Get tenant manager
     * @return TenantManager
     */
    public TenantManager getTenantManager();
    
}
