/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.user.core.tenant;

import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.BundleContext;

import javax.sql.DataSource;
import java.util.Map;

/**
 * this class is used to store the tenant details in ldap server as well
 */

public class HybridLDAPTenantManager extends JDBCTenantManager {

    private static final Log log = LogFactory.getLog(HybridLDAPTenantManager.class);

    public HybridLDAPTenantManager(OMElement omElement, Map<String, Object> properties) throws Exception {
        super(omElement, properties);
    }

    public HybridLDAPTenantManager(DataSource dataSource, String superTenantDomain) {
        super(dataSource, superTenantDomain);
    }

    private void initializeTheExistingPartitions() {
        try {
            Tenant[] existingTenents = getAllTenants();
            LDAPTenantManager ldapTenantManager = getLDAPTenantManager();
            for (Tenant tenant : existingTenents) {
                if (!tenant.getDomain().equals(this.superTenantDomain)) {
                    ldapTenantManager.addPartitionToTenant(tenant);
                }
            }
        } catch (UserStoreException e) {
            log.error("Can not get the existing tenents", e);
        }
    }

    @Override
    public int addTenant(Tenant tenant) throws UserStoreException {
        int tenantID = super.addTenant(tenant);
        tenant.setId(tenantID);
        LDAPTenantManager ldapTenantManager = getLDAPTenantManager();
        ldapTenantManager.addTenant(tenant);
        return tenantID;
    }

    @Override
    public void updateTenant(Tenant tenant) throws UserStoreException {
        super.updateTenant(tenant);
        LDAPTenantManager ldapTenantManager = getLDAPTenantManager();
        ldapTenantManager.updateTenant(tenant);
    }

    @Override
    public void deleteTenant(int tenantId) throws UserStoreException {
        super.deleteTenant(tenantId);
        LDAPTenantManager ldapTenantManager = getLDAPTenantManager();
        ldapTenantManager.deleteTenant(tenantId);
    }

     public void setBundleContext(BundleContext bundleContext) {
        super.setBundleContext(bundleContext);
        // this is a hack we need to do this at the constructor
        // since at the constructor we don't have the bundle context we do this but
        // should happen at the constructor
        initializeTheExistingPartitions();
    }

    private LDAPTenantManager getLDAPTenantManager(){
        ServiceReference serviceReference = this.bundleContext.getServiceReference(LDAPTenantManager.class.getName());
        return (LDAPTenantManager) this.bundleContext.getService(serviceReference);
    }
}
