package org.wso2.carbon.user.mgt.ui;

import javax.activation.DataHandler;

import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.mgt.common.ClaimValue;
import org.wso2.carbon.user.mgt.common.FlaggedName;
import org.wso2.carbon.user.mgt.common.IUserAdmin;
import org.wso2.carbon.user.mgt.common.UIPermissionNode;
import org.wso2.carbon.user.mgt.common.UserAdminException;
import org.wso2.carbon.user.mgt.common.UserStoreInfo;

public class UserAdminClient implements IUserAdmin {

    public static final String USER_STORE_INFO = "org.wso2.carbon.userstore.info";
    private UserAdminStub stub = null;

    private static Log log = LogFactory.getLog(UserAdminClient.class);
        
    public UserAdminClient(String cookie, String url, ConfigurationContext configContext)
            throws java.lang.Exception {
        try {

            stub = new UserAdminStub(configContext, url + "UserAdmin");
            ServiceClient client = stub._getServiceClient();
            Options option = client.getOptions();
            option.setManageSession(true);
            option.setProperty(org.apache.axis2.transport.http.HTTPConstants.COOKIE_STRING, cookie);
        } catch (java.lang.Exception e) {
            handleException(e.getMessage(), e);
        }
    }

    public void addRole(String roleName, String[] userList, String[] permissions)
            throws UserAdminException {
        try {
            stub.addRole(roleName, userList, permissions);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
    }

    public void addUser(String userName, String password, String[] roles, ClaimValue[] claims,
            String profileName) throws UserAdminException {
        try {
            stub.addUser(userName, password, roles, Util.toADBClaimValues(claims), profileName);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
    }

    public void changePassword(String userName, String newPassword) throws UserAdminException {
        try {
            stub.changePassword(userName, newPassword);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }

    }

    public void deleteRole(String roleName) throws UserAdminException {
        try {
            stub.deleteRole(roleName);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }

    }

    public void deleteUser(String userName) throws UserAdminException {
        try {
            stub.deleteUser(userName);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }

    }

    public FlaggedName[] getAllRolesNames() throws UserAdminException {
        try {
            return Util.toCommonFlaggedNames(stub.getAllRolesNames());
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return (new FlaggedName[0]);
    }

    public FlaggedName[] getRolesOfUser(String userName) throws UserAdminException {
        try {
            return Util.toCommonFlaggedNames(stub.getRolesOfUser(userName));
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return (new FlaggedName[0]);
    }

    public FlaggedName[] getUsersOfRole(String roleName, String filter) throws UserAdminException {
        try {
            return Util.toCommonFlaggedNames(stub.getUsersOfRole(roleName, filter));
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return new FlaggedName[0];
    }

    public UserStoreInfo getUserStoreInfo() throws UserAdminException {
        try {
            return Util.toCommonUserStoreInfo(stub.getUserStoreInfo());
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return null;
    }

    public String[] listUsers(String filter) throws UserAdminException {
        try {
            return stub.listUsers(filter);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return new String[0];
    }

    public void updateRolesOfUser(String userName, String[] newUserList) throws UserAdminException {
        try {
            stub.updateRolesOfUser(userName, newUserList);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }

    }

    public void updateUsersOfRole(String roleName, FlaggedName[] userList)
            throws UserAdminException {
        try {
            stub.updateUsersOfRole(roleName, Util.toADBFlaggedNames(userList));
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }

    }

    public UIPermissionNode getAllUIPermissions() throws UserAdminException {
        try {
            return Util.toCommonUIPermissionNode(stub.getAllUIPermissions());
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return null;
    }

    public UIPermissionNode getRolePermissions(String roleName) throws UserAdminException {
        try {
            return Util.toCommonUIPermissionNode(stub.getRolePermissions(roleName));
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
        return null;
    }

    public void setRoleUIPermission(String roleName, String[] rawResources)
            throws UserAdminException {
        try {
            stub.setRoleUIPermission(roleName, rawResources);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
    }

    public void bulkImportUsers(String fileName, DataHandler handler, String defaultPassword)
            throws UserAdminException {
        try {
            stub.bulkImportUsers(fileName, handler, defaultPassword);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
    }
    
    public void changePasswordByUser(String oldPassword, String newPassword)
            throws UserAdminException {
        try {
            stub.changePasswordByUser(oldPassword, newPassword);
        } catch (Exception e) {
            handleException(e.getMessage(), e);
        }
    }

    private String[] handleException(String msg, Exception e)
            throws org.wso2.carbon.user.mgt.common.UserAdminException {
        log.error(msg, e);
        throw new org.wso2.carbon.user.mgt.common.UserAdminException(msg, e);
    }
}
