/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.CarbonConstants;

import java.net.URL;
import java.net.MalformedURLException;

public class TenantUtils {
    
    private static final Log log = LogFactory.getLog(TenantUtils.class);

    /**
     * Obtain the domain name from an OpenID
     * @param openId
     * @return domain name
     */
    public static String getDomainNameFromOpenId(String openId) {
        openId = openId.trim();
        String hostName = null;
        try {
            
            if (openId.indexOf("@")>0){
            	hostName = openId.substring(openId.indexOf("@")+1);
            }
            
            if (!(openId.startsWith("http://") || openId.startsWith("https://"))) {
                openId = "http://" + openId;
            }
            URL url = new URL(openId);
            hostName = url.getHost();

            if (hostName != null && url.getHost().indexOf(".") > 0) {
            	hostName = hostName.substring(url.getHost().indexOf(".")+1);
            }
            
        } catch (MalformedURLException e) {
            log.error("Error extracting domain name from OpenID", e);
        }
        
        if (getHostName().equals(hostName)) {
            hostName = null;
        }
        
        return hostName;
    }

    public static String getTenantDomainFromRequestURL(String requestURI) {
        String domain = null;
        if (requestURI.contains("/" + CarbonConstants.TENANT_AWARE_URL_PREFIX + "/")) {
            int index = requestURI.indexOf("/" + CarbonConstants.TENANT_AWARE_URL_PREFIX + "/");
            int endIndex = requestURI.indexOf("/", index + 3);
            domain = requestURI.substring(index + 3, endIndex);
        }
        return domain;
    }
    
    private static String getHostName() {
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        if (serverConfig.getFirstProperty("HostName") != null) {
            return serverConfig.getFirstProperty("HostName");
        } else {
            return "localhost";
        }
    }   

}

