/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.utils;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.AxisServiceGroup;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.CarbonException;
import org.wso2.carbon.utils.CarbonUtils;
import org.wso2.carbon.utils.ServerConstants;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.security.MessageDigest;
import java.math.BigInteger;

/**
 * A collection of useful utility methods
 */
public class CarbonUtils {

    private static Log log = LogFactory.getLog(CarbonUtils.class);
    private static boolean isServerConfigInitialized;

    public static boolean isAdminConsoleEnabled() {
        boolean enableAdminConsole = false;
        String enableAdminConsoleProp =
                ServerConfiguration.getInstance().getFirstProperty("Management.EnableConsole");
        if (enableAdminConsoleProp != null) {
            enableAdminConsole = Boolean.valueOf(enableAdminConsoleProp);
        }
        return enableAdminConsole;
    }

    /**
     * Get the port corresponding to a particular transport
     *
     * @param configContextService The OSGi ConfigurationContextService
     * @param transport            The transport
     * @return The port corresponding to the <code>transport</code>
     * @throws IllegalStateException If no port is assocaited with the transport, or if this method
     *                               has been called before the ListenerManager has started
     */
    public static int getTransportPort(ConfigurationContextService configContextService,
                                       String transport) {
        return getTransportPort(configContextService.getServerConfigContext(), transport);
    }

    /**
     * Get the port corresponding to a particular transport
     *
     * @param configContext The Axis2 ConfigurationContext
     * @param transport     The transport
     * @return The port corresponding to the <code>transport</code>
     * @throws IllegalStateException If no port is assocaited with the transport, or if this method
     *                               has been called before the ListenerManager has started
     */
    public static int getTransportPort(ConfigurationContext configContext, String transport) {
        return getTransportPort(configContext.getAxisConfiguration(), transport);
    }

    /**
     * Get the port corresponding to a particular transport
     *
     * @param axisConfig The AxisConfiguration
     * @param transport  The transport
     * @return The port corresponding to the <code>transport</code>
     * @throws IllegalStateException If no port is assocaited with the transport, or if this method
     *                               has been called before the ListenerManager has started
     */
    public static int getTransportPort(AxisConfiguration axisConfig, String transport) {
        String transportPort = System.getProperty("carbon." + transport + ".port");
        if (transportPort == null) {
            Parameter portParam = axisConfig.getTransportIn(transport).getParameter("port");
            if (portParam != null) {
                transportPort = (String) portParam.getValue();
            } else {
                throw new IllegalStateException(transport + " port has not been set yet. " +
                        "Most probably the ListenerManager has not" +
                        " been started yet or the this transport does not " +
                        "have a port associated with it. You can wait until " +
                        "the org.apache.axis2.engine.ListenerManager OSGi " +
                        "service is available & retry.");
            }
        }
        return Integer.parseInt(transportPort);
    }

    public static String getAxis2Xml() {
        String axis2XML = ServerConfiguration.getInstance().
                getFirstProperty("Axis2Config.ConfigurationFile");
        if (axis2XML == null) {
            axis2XML = System.getProperty(Constants.AXIS2_CONF);
        }
        return axis2XML;
    }

    public static String getCarbonXml() {
        String confFolder = getCarbonConfigDirPath();
        return confFolder + File.separator + ServerConstants.CONF_FILE_NAME_CARBON_XML;
    }

    public static String getMgtTransportXml() {
        String transportXml = System.getProperty(ServerConstants.TRANSPORTS_XML_PATH);
        if (transportXml==null)
            transportXml=getCarbonConfigDirPath() + File.separator + ServerConstants.CONF_FILE_NAME_MGT_TRANSPORT_XML;
        return transportXml;
    }


    public static String getRegistryRepoPath() {
        return System.getProperty("registryRepoPath");
    }

    public static String getCarbonHome() {
        String carbonHome = System.getProperty(ServerConstants.CARBON_HOME);
        if (carbonHome == null) {
            carbonHome = System.getenv(CarbonConstants.CARBON_HOME_ENV);
        }
        return carbonHome;
    }

    public static String getTomcatDocsDir() {
        String docPath = getCarbonHome() + File.separator + "docs";
        return docPath;
    }

    public static String getCatalinaHome() {
        String catalinaHome = System.getProperty(ServerConstants.CATALINA_HOME);
        if (catalinaHome == null) {
            catalinaHome = getCarbonHome() + File.separator + "lib" + File.separator + "tomcat";
        }
        return catalinaHome;
    }

    public static String getCarbonTenantsDirPath() {
        String carbonTenantsDirPath = System.getProperty(ServerConstants.CARBON_TENANTS_DIR_PATH);
        if (carbonTenantsDirPath == null) {
            carbonTenantsDirPath = System.getenv(CarbonConstants.CARBON_TENANTS_DIR_PATH_ENV);
        }
        if (carbonTenantsDirPath == null) {
            carbonTenantsDirPath = getCarbonHome() + File.separator + "repository" +
                    File.separator + "tenants";
        }
        return carbonTenantsDirPath;
    }

    public static String getTomcatWebappDirPath() {
        String tomcatWebappDirPath = System.getProperty(ServerConstants.TOMCAT_WEBAPP_DIR_PATH);
        if (tomcatWebappDirPath == null) {
            tomcatWebappDirPath=getCarbonHome() + File.separator + "webapps";
        }
        return tomcatWebappDirPath;
    }

    public static String getCarbonConfigDirPath() {
        String carbonConfigDirPath = System.getProperty(ServerConstants.CARBON_CONFIG_DIR_PATH);
        if (carbonConfigDirPath == null) {
            carbonConfigDirPath = System.getenv(CarbonConstants.CARBON_CONFIG_DIR_PATH_ENV);
            if (carbonConfigDirPath == null) {
                return getCarbonHome() + File.separator + "repository" + File.separator + "conf";
            }
        }
        return carbonConfigDirPath;
    }

    public static String getCarbonLogsPath() {
        String carbonLogsPath = System.getProperty(ServerConstants.CARBON_LOGS_PATH);
        if (carbonLogsPath == null) {
            carbonLogsPath = System.getenv(CarbonConstants.CARBON_LOGS_PATH_ENV);
            if (carbonLogsPath == null) {
                return getCarbonHome() + File.separator + "repository" + File.separator + "logs";
            }
        }
        return carbonLogsPath;
    }

    public static String getCarbonRepositoryDir() {
        String carbonRepo = System.getenv("CARBON_REPOSITORY");
        if(carbonRepo == null){
            carbonRepo = System.getProperty(ServerConstants.CARBON_REPO);
        }
        if(carbonRepo == null){
            carbonRepo = getCarbonHome() + File.separator + ServerConstants.FOLDER_NAME_REPOSITORY;
        }
        return carbonRepo;
    }
    
    public static String getAxis2DeployerDir() {
        return getCarbonOSGIRepoDir() + File.separator + ServerConstants.FOLDER_NAME_AXIS2_DEPLOYERS;
    }
    
    public static String getExtensionsDir() {
        return getCarbonOSGIRepoDir() + File.separator + ServerConstants.FOLDER_NAME_EXTENSIONS;
    }

    public static String getOSGILibDir() {
        return getCarbonOSGIRepoDir() + File.separator + ServerConstants.FOLDER_NAME_OSGI_LIB;
    }

    public static String getOSGIMediatorsDir() {
        return getCarbonOSGIRepoDir() + File.separator + ServerConstants.FOLDER_NAME_OSGI_MEDIATORS;
    }
        
    public static String getCarbonOSGIRepoDir() {
        String osgiRepo = System.getProperty(ServerConstants.CARBON_OSGI_REPO);
        if (osgiRepo == null) {
            osgiRepo = getCarbonRepositoryDir() + File.separator + ServerConstants.FOLDER_NAME_COMPONENTS;
        }
        return osgiRepo;
    }

    public static String getComponentsRepo() {
        String componentsRepo = System.getProperty(ServerConstants.COMPONENT_REP0);
        if (componentsRepo == null) {
            componentsRepo = System.getenv(CarbonConstants.COMPONENT_REP0_ENV);
            if (componentsRepo == null) {
                return getCarbonHome() + File.separator + ServerConstants.FOLDER_NAME_REPOSITORY + File.separator + ServerConstants.FOLDER_NAME_COMPONENTS;
            }
        }
        return componentsRepo;
    }

    public static String getAxis2Repo() {
        String axis2Repo = System.getProperty(ServerConstants.AXIS2_REPO);
        if (axis2Repo == null) {
            axis2Repo = System.getenv(CarbonConstants.AXIS2_REPO_ENV);
        }
        return axis2Repo;
    }

    public static String getRegistryXMLPath() {
        String configPath = System.getProperty(ServerConstants.REGISTRY_XML_PATH);
        if (configPath == null) {
            configPath = getCarbonConfigDirPath() + File.separator + ServerConstants.CONF_FILE_NAME_REGISTRY_XML;
        }
        return configPath;
    }

    public static String getUserMgtXMLPath() {
            String configPath = System.getProperty(ServerConstants.USER_MGT_XML_PATH);
        if (configPath == null) {
            configPath = getCarbonConfigDirPath() + File.separator + ServerConstants.CONF_FILE_NAME_USER_MGT_XML;
        }
        return configPath;
    }
//    public static String getCarbonHome() {
//        String carbonHome = System.getProperty(ServerConstants.CARBON_HOME);
//        if (carbonHome == null) {
//            carbonHome = System.getenv("CARBON_HOME");
//        }
//        return carbonHome;
//    }

    /**
     * Check whther the specified Strin corresponds to a URL
     *
     * @param location The String to be checked
     * @return true - if <code>location</code> is a URL, false - otherwise
     */
    public static boolean isURL(String location) {
        try {
            new URL(location);
            return true;
        } catch (MalformedURLException e) {
            return false;
        }
    }

    /**
     * Get the last updated time of the service artifact corresponding to AxisServiceGroup
     * serviceGroup
     *
     * @param serviceGroup The service group whose last updated time we need to get
     * @return The last updated time of the service artifact corresponding to AxisServiceGroup
     *         serviceGroup. Will return null if a Axis2 URL repository is in use.
     */
    public static Long lastUpdatedTime(AxisServiceGroup serviceGroup) {
        String axis2Repo = ServerConfiguration.getInstance().
                getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
        if (isURL(axis2Repo)) { // We do not support retrieving the timestamps of URLs
            return null;
        }

        Long lastUpdated = null;
        String fileName = "";
        for (Iterator serviceIter = serviceGroup.getServices(); serviceIter.hasNext();) {
            AxisService axisService = (AxisService) serviceIter.next();
            URL fn = axisService.getFileName();
            if (fn != null) {
                fileName = fn.getPath();
            }
            if ((fileName != null) && (fileName.trim().length() != 0)) {
                File file = new File(fileName);
                lastUpdated = file.lastModified();
                break;
            }
        }
        return lastUpdated;
    }

    /**
     * Get the MD5 hash value of the service artifact corresponding to AxisServiceGroup
     * serviceGroup
     *
     * @param serviceGroup The service group whose last updated time we need to get
     * @return The MD5 hash of the service artifact corresponding to AxisServiceGroup
     *         serviceGroup. Will return null if a Axis2 URL repository is in use.
     */
    public static String computeServiceHash(AxisServiceGroup serviceGroup) {
        String axis2Repo = ServerConfiguration.getInstance().
                getFirstProperty(ServerConfiguration.AXIS2_CONFIG_REPO_LOCATION);
        if (isURL(axis2Repo)) { // We do not support retrieving the timestamps of URLs
            return null;
        }

        // If there is a parameter to always force a new service addtion, we don't need hash
        // TODO - Fix handleExistingServiceInit method and remove this parameter
        Parameter param = serviceGroup.getParameter(CarbonConstants.FORCE_EXISTING_SERVICE_INIT);
        if (param != null && "true".equals(param.getValue().toString())) {
            return null;
        }

        String hashValue = null;
        String fileName = "";
        for (Iterator serviceIter = serviceGroup.getServices(); serviceIter.hasNext();) {
            AxisService axisService = (AxisService) serviceIter.next();
            URL fn = axisService.getFileName();
            if (fn != null) {
                fileName = fn.getPath();
            }
            if ((fileName != null) && (fileName.trim().length() != 0)) {
                File file = new File(fileName);
                try {
                    hashValue = getMD5(getBytesFromFile(file));
                } catch (CarbonException ignore) {
                    // If some error occures in calculating the hash, just ignore it..
                }
                break;
            }
        }
        return hashValue;
    }

    /**
     * Get the port of the command listener. A server socket is opened on this port to listen
     * to incoming commands
     *
     * @return The command listener port
     */
    public static int getCommandListenerPort() {
        int serverPort = -1;
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        String port = serverConfig.getFirstProperty("Ports.CommandListener");
        if (port == null) {
            port = serverConfig.getFirstProperty("CommandListener.Port");
        }
        if (port != null) {
            serverPort = Integer.parseInt(port);
        }
        return serverPort;
    }

    /**
     * Register a faulty service
     *
     * @param serviceName          Name of the faulty service
     * @param serviceType          Service type of the faulty service
     * @param configurationContext ConfigurationContext
     * @throws AxisFault If an error occurs while creating the service type Axis2 parameter
     */
    public static void registerFaultyService(String serviceName,
                                             String serviceType,
                                             ConfigurationContext configurationContext)
            throws AxisFault {
        AxisService service = new AxisService(serviceName);
        if (serviceType != null) {
            Parameter serviceTypeParam = new Parameter(ServerConstants.SERVICE_TYPE,
                    serviceType);
            service.addParameter(serviceTypeParam);
        }

        Map<String, AxisService> faultyServicesMap =
                (Map<String, AxisService>) configurationContext.
                        getPropertyNonReplicable(CarbonConstants.FAULTY_SERVICES_MAP);
        if (faultyServicesMap == null) {
            faultyServicesMap = new HashMap<String, AxisService>();
            configurationContext.setNonReplicableProperty(CarbonConstants.FAULTY_SERVICES_MAP,
                    faultyServicesMap);
        }
        faultyServicesMap.put(serviceName, service);
    }

    /**
     * Get a faulty service
     *
     * @param serviceName          Name of the faulty service
     * @param configurationContext ConfigurationContext
     * @return AxisService object corresponding to the <code>serviceName</code>
     */
    public static AxisService getFaultyService(String serviceName,
                                               ConfigurationContext configurationContext) {
        Map<String, AxisService> faultyServicesMap =
                (Map<String, AxisService>) configurationContext.
                        getPropertyNonReplicable(CarbonConstants.FAULTY_SERVICES_MAP);
        if (faultyServicesMap != null) {
            return faultyServicesMap.get(serviceName);
        }
        return null;
    }

    public static boolean isRemoteRegistry() throws Exception {
        ServerConfiguration serverConfig = getServerConfiguration();
        String isRemoteRegistry =
                serverConfig.getFirstProperty("Registry.Type");
        return (isRemoteRegistry != null && isRemoteRegistry.equalsIgnoreCase("remote"));
    }

    public static String getCarbonRepository() {
        ServerConfiguration serverConfig = getServerConfiguration();
        return serverConfig.getFirstProperty("Axis2Config.RepositoryLocation"); //TODO: Change to Carbon.Repository in carbon.xml
    }

    public static ServerConfiguration getServerConfiguration() {
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        if (!isServerConfigInitialized) {
            String carbonXml = CarbonUtils.getCarbonXml();
            File carbonXML = new File(carbonXml);
            InputStream inSXml = null;
            try {
                inSXml = new FileInputStream(carbonXML);
                serverConfig.init(inSXml);
                isServerConfigInitialized = true;
            } catch (Exception e) {
                log.error("Cannot read file " + carbonXml, e);
            } finally {
                if (inSXml != null) {
                    try {
                        inSXml.close();
                    } catch (IOException e) {
                        log.warn("Cannot close file " + carbonXml, e);
                    }
                }
            }
        }
        return serverConfig;
    }

    /**
     * Check if this is a ChildNode or Not
     *
     * @return true if the instance is a child node in multiple instance scenario
     */
    public static boolean isMasterNode() {
        if ("true".equals(System.getProperty("instance")) && "true".equals(System.getProperty("master"))) {
            return true;
        }
        return false;
    }

    /**
     * Check if this is an Instance started by a Java exec
     *
     * @return true if this is an instance started by Java exec
     */
    public static boolean isChildNode() {
        if ("true".equals(System.getProperty("instance"))) {
            return true;
        }
        return false;
    }

    /**
     * Check whether this is the multiple Instance scenario- means started the server with -n arg
     *
     * @return true if the server started with -n argument
     */
    public static boolean isMultipleInstanceCase() {
        if (System.getProperty("instances.value") != null) {
            return true;
        }
        return false;
    }

    /**
     * Check whether this is the a node which should readonly
     *
     * @return true if the server started with -n argument
     */
    public static boolean isReadOnlyNode() {
        if (isChildNode() && !isMasterNode()) {
            return true;
        }
        return false;
    }

    /**
     * Returns the contents of the file in a byte array.
     *
     * @param file the file the to read
     * @return the content of the file
     * @throws CarbonException throws if the operation failed.
     */
    public static byte[] getBytesFromFile(File file) throws CarbonException {
        InputStream is;
        try {
            is = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            throw new CarbonException("Error creating the file input stream", e);
        }
        long length = file.length();

        // to ensure that file is not larger than Integer.MAX_VALUE.
        if (length > Integer.MAX_VALUE) {
            // File is too large
            throw new CarbonException("File to read is too large. File name: " +
                    file.getName() + ". File length limit: " + Integer.MAX_VALUE);
        }

        // byte array to keep the data
        byte[] bytes = new byte[(int)length];

        int offset = 0;
        int numRead;
        try {
            while (offset < bytes.length
                    && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
                offset += numRead;
            }
        } catch (IOException e) {
            throw new CarbonException("Error in reading file. File name: " + file.getName());
        }

        if (offset < bytes.length) {
            throw new CarbonException("Couldn't read the entire file. " +
                    "File name: " + file.getName());
        }
        try {
            is.close();
        } catch (IOException e) {
            throw new CarbonException("Error in closing the file input stream. " +
                    "File name: " + file.getName());
        }
        return bytes;
    }

    public static String getMD5(byte[] content) {
        MessageDigest m;
        try {
            m = MessageDigest.getInstance("MD5");
        } catch (Exception e) {
            log.error("Error while generating the MD5", e);
            return null;
        }
        m.update(content, 0, content.length);

        return new BigInteger(1,m.digest()).toString(16);
    }

}
