/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */
package org.wso2.carbon.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.utils.dbcreator.DatabaseCreator;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.*;
import java.util.Iterator;

public class DBUtils {
    private static final Log log = LogFactory.getLog(DBUtils.class);

    /**
     * Converts upper case column name to lower case to match PostgreSQL.
     *
     * @param dbProductName - eg: "MySQL,Oracle,PostgreSQL". Obtained using Connection.getMetaData().getDatabaseProductName();
     * @param columnName    - table column name to be converted
     * @return
     */
    public static String getConvertedAutoGeneratedColumnName(String dbProductName,
                                                             String columnName) {
        String autoGeneratedColumnName = columnName;

        if ("PostgreSQL".equals(dbProductName)) {
            autoGeneratedColumnName = columnName.toLowerCase();
            if (log.isDebugEnabled()) {
                log.debug("Database product name is PostgreSQL. Converting column name "
                        + columnName + " to lowercase ("
                        + autoGeneratedColumnName + ").");
            }
        }
        return autoGeneratedColumnName;
    }

    /**
     * get the registry.xml database type
     *
     * @return database name from the registry.xml
     *
     * @throws Exception
     */
    public static String getRegistryDBConfig() throws Exception {
        String path = CarbonUtils.getRegistryXMLPath();
        if (path != null) {
            File file = new File(path);
            Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(file);
            NodeList dbConfig = doc.getElementsByTagName("dbConfig");
            Element inner = (Element)dbConfig.item(0);
            NodeList urlFinder = inner.getElementsByTagName("url");
            String urlValue = urlFinder.item(0).getTextContent();
            return DatabaseCreator.getDatabaseType(urlValue);
        }else {
                log.error("Cannot Find registry.xml in path: " + path);
                return null;
            }
    }

    /**
     * get the user-mgt.xml database type
     *
     * @return database name from the user-mgt.xml
     *
     * @throws Exception
     */
    public static String getUserManagerDBConfig() throws Exception{
        String path = CarbonUtils.getUserMgtXMLPath();
         if (path != null) {
             File file = new File(path);
             Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(file);
             NodeList dbConfig = doc.getElementsByTagName("dbConfig");
             Element inner = (Element)dbConfig.item(0);
             NodeList urlFinder = inner.getElementsByTagName("url");
             String urlValue = urlFinder.item(0).getTextContent();
             return DatabaseCreator.getDatabaseType(urlValue);
        }else {
                log.error("Cannot Find registry.xml in path: " + path);
                return null;
            }
    }
}